/*
 *	WGWetsCmdWrappar.m
 *		・wgetとプロセス間通信を行い、結果をWindowControllerに返す。 
 *
 *  Copyright (c) 2001- Tomonori Tomatsu. All rights reserved.
 *
 *  修正履歴
 *  0.20a2
 *  2002/03/25 wgetのoutputの読み方を、Timerベースから、
 *             NSFileHandleのNSFileHandleReadCompletionNotificationベースで、取得するようにした。（Apple社のサンプルMoriarityを参考。） 
 *  2003/09/11 コマンドを内包するようにした。 
 *
 */
#import "WGWetsCmdWrappar.h"

@implementation WGWetsCmdWrappar
///////////////////////////////////////////////////
// 		 初期化と解放			 //
///////////////////////////////////////////////////
/*===================================
    init 
=====================================*/
- (id)init
{
    [ super init ];

    mNowDownloading = NO;
    mTask = [ [ NSTask alloc ] init ];
    mPipe = [ NSPipe pipe ];
    [ mPipe retain ];
    mReadHandle = [ mPipe fileHandleForReading ];		// mPipeにretainされている。 
    
    // Observer登録 
    [ [ NSNotificationCenter defaultCenter ] addObserver : self 
						selector : @selector( checkTerminationStatus : ) 
						    name : NSTaskDidTerminateNotification 
						  object : mTask ];
    [ [ NSNotificationCenter defaultCenter ] addObserver : self 
						selector : @selector( getData : ) 
						    name : NSFileHandleReadCompletionNotification 
						  object : mReadHandle ];		// mTaskじゃなかったけ？ 
    return self;
}

/*===================================
    dealloc 
=====================================*/
 - (void)dealloc
{
    [ [ NSNotificationCenter defaultCenter ] removeObserver : self ];
    [ mPipe release ];
    [ mTask release ];
    mPipe = nil;
    mTask = nil;
    mReadHandle = nil;

    [ super dealloc ];
}

//////////////////////////////////////////////////
// 		インターフェイスと実装 		//
//////////////////////////////////////////////////
/*===================================
    startDownload
	ダウンロードスタート  
=====================================*/
- (BOOL)startDownload : (NSMutableArray*)inArgs
{
	NSString*	theCommandPath;
	
    if ( mNowDownloading ) return NO;		// あり得ないはず。 
    
    mNowDownloading = YES;

    // Setting Args.
    [ mTask setArguments : inArgs ];

    // Setting Path.
    [ mTask setCurrentDirectoryPath : NSHomeDirectory() ];
//	if ( 0x1010 <= [ self systemVersion ]  )
	if ( 0 )
	{
		theCommandPath = [ NSString stringWithFormat : @"%@/wget",
														[ [ NSBundle mainBundle ] sharedSupportPath ] ];
	}
	else
	{
		theCommandPath = @"/usr/bin/wget";
	}
    [ mTask setLaunchPath : theCommandPath ];
    // Setting wget output.
    [ mTask setStandardOutput : mPipe ];
    [ mTask setStandardError : mPipe ];

    [ mReadHandle readInBackgroundAndNotify ];
    // Launch.
    [ mTask launch ];
    
    return YES;
}

/*===================================
    stopDownload
	ダウンロードストップ   
=====================================*/
- (void)stopDownload
{
    [ mTask terminate ];
    [ mPipe release ];			// 仮にもう一度、releaseされても大丈夫のはず。 
    [ mTask release ];
    
    mPipe = nil;
    mTask = nil;
    mReadHandle = nil;
}

//////////////////////////////////////////////////
// 		局所メソッド	 		//
//////////////////////////////////////////////////
/*===================================
    seeOutput
	標準エラーを見る    
=====================================*/
- (void)seeOutput
{
    NSString*	theDataStr;
    NSData*	theData = nil;

    if ( ( theData = [ mReadHandle availableData ] ) && ( [ theData length ] ) ) {
	theDataStr  = [ NSString stringWithFormat : @"%s", [ theData bytes ] ];
	[ (WGWindowController*)mWindowCtlr getResponseWithString : theDataStr ];
    }
}

//////////////////////////////////////////////////
// 	NSFileHandle's Notification		//
//////////////////////////////////////////////////
/*===================================
    getData
	データを取得する     
=====================================*/
- (void)getData : (NSNotification *)inNotification
{
    NSString*	theDataStr;
    NSData*	theData = [ [ inNotification userInfo ] objectForKey : NSFileHandleNotificationDataItem ];

    if ( [ theData length ] ) {
		theDataStr  = [ NSString stringWithCString :  [ theData bytes ] length : [ theData length ] ];
		[ (WGWindowController*)mWindowCtlr getResponseWithString : theDataStr ];
		[ mReadHandle readInBackgroundAndNotify ];					// Don't Forget!  
    } else {
	// 終了。後に、NSTaskDidTerminateNotificationがポストされる。 
    }
}

//////////////////////////////////////////////////
// 		NSTask's Notification		//
//////////////////////////////////////////////////
/*===================================
    checkTerminationStatus
	mTaskのnotifier 
=====================================*/
- (void)checkTerminationStatus : (NSNotification *)inNotification
{
    [ self seeOutput ];		// log表示で、タイムラグを矯正 

    [ (WGWindowController*)mWindowCtlr getResponseWithTerminationStatus : [ [ inNotification object ] terminationStatus ] ];
}

/*==========================================================================================
- (long) systemVersion

- 履歴 - 
	02/10/30 Tomatsu	YuBurnerから、持ってきた  

	==========================================================================================*/
- (long) systemVersion
{
	OSErr		theErr;
	long		theSysVer = 0;
	
	theErr = Gestalt( gestaltSystemVersion, &theSysVer );
	if ( noErr != theErr )
	{
		theSysVer = 0;
	}
	
	return theSysVer;
}

@end
