/*
*	WGWindowController.m
*		・windowController
*		・MediatorパターンのMediator(ConcreteMediator)っぽいかしら？ 
*		・WGWetsCmdWrapparとは、Template Methodパターンでやった方がいいのでしょうか？ 
*		・大きいので、分割を検討すべし。 
*
*  Copyright (c) 2001 Tomonori Tomatsu. All rights reserved.
*/
#import "WGWindowController.h"

@implementation WGWindowController
///////////////////////////////////////////////////
// 		 初期化と解放			 //
///////////////////////////////////////////////////
/*===================================
    init
=====================================*/
- (id)init
{
    [ super init ];
    mNowDownloading = NO;
    mArgs = [ [ NSMutableArray alloc ] init ];
    
    [ [ NSNotificationCenter defaultCenter ] addObserver : self 
						selector : @selector( openLocation: ) 
						    name : kWGOpenLocationDropedNotification 
						  object : self ];
    
    return self;
}

/*================================
    awakeFromNib
==================================*/
- (void)awakeFromNib 
{
    [ mWGPreference retain ];				// とりあえず、呼んでおくが・・・。 
    [ (WGDocument*)mWGDocument addMainWindowController : self ];

    [ log setHasHorizontalScroller : YES ];
    [ log setHasVerticalScroller : YES ];
    [ log setAutoresizesSubviews : YES ];
    [ log setAutoresizingMask : (NSViewWidthSizable | NSViewHeightSizable) ];
    [ [ log contentView ] setAutoresizesSubviews : YES ];

    [ mLogTextView setHorizontallyResizable : YES ];
    [ mLogTextView setVerticallyResizable : YES ];
    [ mLogTextView setAutoresizingMask : NSViewNotSizable ];
    [ mLogTextView setMaxSize : NSMakeSize(1.0e7, 1.0e7) ];

    [ [ NSNotificationCenter defaultCenter ] postNotificationName: kWGGiveMeOpenLocationNotification object : self ];
}

/*===================================
    dealloc    
=====================================*/
- (void)dealloc
{
    [ mArgs release ];
    [ mWGPreference release ];				// とりあえず、呼んでおくが・・・。 
    
    [ [ NSNotificationCenter defaultCenter ] removeObserver : self ];

    [ super dealloc ];
}

///////////////////////////////////////////////////
// 		 action受け付け			 //
///////////////////////////////////////////////////
/*===================================
    downloadAction
	Getボタン押下  
=====================================*/
- (IBAction)downloadAction : (id)sender
{
    NSBundle*	theBundle = [ NSBundle mainBundle ];
    NSString*	theAlertStr = [ theBundle localizedStringForKey : @"Alert" value : @"Alert" table : nil ];
    NSString*	theOKStr = [ theBundle localizedStringForKey : @"OK" value : @"OK" table : nil ];
    NSString*	theCancelStr = [ theBundle localizedStringForKey : @"Cancel" value : @"Cancel" table : nil ];
    NSString*	theStopMsgStr = [ theBundle localizedStringForKey : @"Now Downloading...\nWould you stop it?" value : @"Now Downloading...\nWould you stop it?" table : nil ];
    NSString*	theNoInputMsgStr = [ theBundle localizedStringForKey : @"You must input URL." value : @"You must input URL." table : nil ];

    if ( mNowDownloading ) {
	NSBeginAlertSheet( theAlertStr, theOKStr, theCancelStr, nil, [ self window ], self, 
			   @selector( didEndShouldCloseSheetStopDownload : returnCode : contextInfo : ), NULL, nil,
			   theStopMsgStr, nil );
    } else {
	if ( [ [ url stringValue ] isEqualToString : @"" ] ) {
	    NSBeginAlertSheet( theAlertStr, theOKStr, nil, nil, [ self window ], self, nil, NULL, nil, theNoInputMsgStr, nil);
	} else {
	    mNowDownloading = YES;
	    [ [ NSNotificationCenter defaultCenter ] postNotificationName: kWGNowStartDownloadNotification object : self ];
	    [ self startDownload ];
	}
    }
}

/*===================================
    recursiveAction    
=====================================*/
- (IBAction) recursiveAction : (id)sender
{
}

/*===================================
    mirrorAction    
=====================================*/
- (IBAction)mirrorAction : (id)sender
{
}

//////////////////////////////////////////////////
// 	WGWetsCmdWrapparに対する口			//
//////////////////////////////////////////////////
/*===================================
    getResponseWithString
	wgetよりのレスポンスを表示する    
=====================================*/
- (void)getResponseWithString : (NSString*)inStdErr
{
#if 0
    NSMutableString* newString;

    newString = [ NSMutableString stringWithString : [ mLogTextView string ] ];
    [ newString appendString : @"\n" ];
    [ newString appendString : inStdErr ];
    [ mLogTextView setString : newString ];
#else
    [ mLogTextView setEditable : YES ];
    [ mLogTextView insertText : inStdErr ];
    [ mLogTextView setEditable : NO ];
#endif

    // 何故か自動で変わらないので。。。 
    [ mLogTextView setFrameSize : NSMakeSize(1.0e7, 1.0e7) ];

    [ log setNeedsDisplay : YES ];
}

/*===================================
    getResponseWithTerminationStatus
	wgetよりの終了レスポンスを表示する    
=====================================*/
- (void)getResponseWithTerminationStatus : (int)inTerminationStatus
{
    NSBundle*	theBundle = [ NSBundle mainBundle ];
    NSString*	theENDStr = [ theBundle localizedStringForKey : @"END" value : @"END" table : nil ];
    NSString*	theDownloadENDStr = [ theBundle localizedStringForKey : @"Download end." value : @"Download end." table : nil ];
    NSString*	theOKStr = [ theBundle localizedStringForKey : @"OK" value : @"OK" table : nil ];
    NSString*	theTerminationStatusIsStr = [ theBundle localizedStringForKey : @"TerminationStatus is " value : @"TerminationStatus is " table : nil ];
    NSString*	theForDetailsPleaseSeeALogStr = [ theBundle localizedStringForKey : @".\nFor details, please see a log." value : @".\nFor details, please see a log." table : nil ];
    NSMutableString* theMsg;

    // ダウンロード終了をポスト 
    [ [ NSNotificationCenter defaultCenter ] postNotificationName : kWGNowTerminateDownloadNotification
							   object : self
							 userInfo : nil ];
    mNowDownloading = NO;
    [ getBtn setTitle : theENDStr ];
    [ getBtn setEnabled : NO ];

    // 気休めメッセージ "TerminationStatus is %d.\nFor details, please see a log."
    theMsg = [ NSMutableString stringWithString : theTerminationStatusIsStr ];
    [ theMsg appendFormat: @"%d", inTerminationStatus ];
    [ theMsg appendString : theForDetailsPleaseSeeALogStr ];

    NSBeginAlertSheet( theDownloadENDStr, theOKStr, nil, nil, [ self window ],
		       self, nil, NULL, nil, theMsg, nil );
}

//////////////////////////////////////////////////
// 	WGDocumentに対しての口			//
//////////////////////////////////////////////////
/*===================================
    windowShouldClose
	windowを閉じてよいか？     
=====================================*/
- (BOOL) windowShouldClose : (NSWindow*)sender
{
    NSBundle*	theBundle = [ NSBundle mainBundle ];
    NSString*	theAlertStr = [ theBundle localizedStringForKey : @"Alert" value : @"Alert" table : nil ];
    NSString*	theOKStr = [ theBundle localizedStringForKey : @"OK" value : @"OK" table : nil ];
    NSString*	theCancelStr = [ theBundle localizedStringForKey : @"Cancel" value : @"Cancel" table : nil ];
    NSString*	theStopItAndCloseWindowStr = [ theBundle localizedStringForKey : @"Now Downloading...\nWould you stop it and close this window?" value : @"Now Downloading...\nWould you stop it and close this window?" table : nil ];

    if ( NO == mNowDownloading ) return YES;

    NSBeginAlertSheet( theAlertStr, theOKStr, theCancelStr, nil, [ self window ], self,
		       @selector( didEndShouldCloseSheetCanCloseWindow : returnCode : contextInfo : ),
		       NULL, sender, theStopItAndCloseWindowStr, nil );
    
    return NO;
}

/*===================================
    printlog
	logをプリント     
=====================================*/
- (void)printlog : (id)sender
{
    [mLogTextView print:sender];
}

#if 0
/*===================================
    openLocation
	Locationをセット     
=====================================*/
- (void)openLocation : (NSString*)path
{
    [ url setStringValue : path ];
}
#endif

//////////////////////////////////////////////////
// 		Notification			//
//////////////////////////////////////////////////
/*===================================
    openLocation
	Locationをセットする   
=====================================*/
- (void)openLocation : (NSNotification *)inNotification
{
    if( [ [ inNotification object ] isEqual : self ] ) {
	[ url setStringValue : [ [ inNotification userInfo ] objectForKey : kWGOpenLocationKey ] ];
    }
}

//////////////////////////////////////////////////
// 		局所メソッド			//
//////////////////////////////////////////////////
/*===================================
    startDownload
	ダウンロードを開始させる 
=====================================*/
- (void)startDownload
{
    NSBundle*	theBundle = [ NSBundle mainBundle ];
    NSString*	theStopStr = [ theBundle localizedStringForKey : @"Stop" value : @"Stop" table : nil ];

    // UIをいじる 
    [ getBtn setTitle : theStopStr ];
    [ mLogTextView setString : @"Download Start.\n" ];
    [ mLogTextView setSelectable : NO ];
    [ [ self window ] setTitle: [ url stringValue ] ];

    [ self setupArgs ];
    [ mWgetWrapper startDownload : (NSMutableArray*)mArgs ];
}

/*===================================
    stopDownload
	ダウンロードを終了させる 
=====================================*/
- (void)stopDownload
{
    // UIをいじる 
    //[ getBtn setTitle : @"get" ];
    [ mLogTextView setSelectable : YES ];

    mNowDownloading = NO;
    [ mWgetWrapper stopDownload ];

}

/*===================================
    didEndShouldCloseSheetStopDownload:returnCode:contextInfo
	ストップボタン押下時のSheetより呼ばれる   
=====================================*/
- (void)didEndShouldCloseSheetStopDownload : (NSWindow*)sheet returnCode : (int)returnCode contextInfo : (void*)contextInfo
{
    if ( NSAlertDefaultReturn == returnCode && mNowDownloading ) {
        [ self stopDownload ];
    }
}

/*===================================
    didEndShouldCloseSheetCanCloseWindow:returnCode:contextInfo
	Windowを閉じる際の確認Sheetより呼ばれる   
=====================================*/
- (void)didEndShouldCloseSheetCanCloseWindow : (NSWindow*)sheet returnCode : (int)returnCode contextInfo : (void*)contextInfo
{
    if ( NSAlertDefaultReturn == returnCode && mNowDownloading ) {
        [ self stopDownload ];	
	[ (NSWindow*)contextInfo close ];
    }
}

/*===================================
    setupArgs
	引数を作成する  
=====================================*/
- (void)setupArgs
{
    NSMutableString* tempString;

    // WGPreferenceより受け取る。 
    [ mArgs addObjectsFromArray : [ mWGPreference preferenceArgs ] ];
    // 取得設定 
    if ( NSOnState == [ ow_NoClobber state ] ) {
	tempString = [ NSMutableString stringWithString : @"-nc" ];
	[mArgs addObject : tempString ];
    } else if ( NSOnState == [ ow_UseTimeStamp state ] ) {
	tempString = [ NSMutableString stringWithString : @"-N" ];
	[ mArgs addObject : tempString ];
    }
    // Quota
    tempString = [ NSMutableString stringWithFormat : @"%@%dm", @"-Q", [ quota intValue ] ];
    [ mArgs addObject : tempString ];
    // レジューム 
    if ( NSOnState == [ continueBtn state ] ) {
	tempString = [ NSMutableString stringWithString : @"-c" ];
	[ mArgs addObject : tempString ];
    }
    // 再起 
    if ( NSOnState == [ recursive state ] ) {
	tempString = [ NSMutableString stringWithString : @"-r" ];
	[ mArgs addObject : tempString ];
	// 再起深度 
	tempString = [ NSMutableString stringWithFormat : @"%@%d", @"-l", [ depth intValue ] ];
	[ mArgs addObject : tempString ];
	// 相対リンクに変更する 
	if ( NSOnState == [ cnvtToRelatvLinks state ] ) {
	    tempString = [ NSMutableString stringWithString : @"-k" ];
	    [ mArgs addObject : tempString ];
	}
	// 違うホストも巡る  
	if ( NSOnState == [ spanHost state ] ) {
	    tempString = [ NSMutableString stringWithString : @"-H" ];
	    [ mArgs addObject : tempString ];
	}
	// FTPも対象にする  
	if ( NSOnState == [ followFTP state ] ) {
	    tempString = [ NSMutableString stringWithString : @"-f" ];
	    [ mArgs addObject : tempString ];
	}
	// 相対リンクのみ 
	if ( NSOnState == [ onlyRelative state ] ) {
	    tempString = [ NSMutableString stringWithString : @"-L" ];
	    [ mArgs addObject : tempString ];
	}
    }

    // ロケーション 
    [ mArgs addObject : [ url stringValue ] ];
}

@end
